@@ -1,7 +1,10 @@
 import datetime
+import logging
+from abc import ABC, abstractmethod
+from contextlib import contextmanager
 from typing import Any, Dict, List, Optional
 
-from sqlalchemy import create_engine, desc, func
+from sqlalchemy import TIMESTAMP, create_engine, desc, func
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import Query, Session, scoped_session, sessionmaker
 
@@ -16,63 +19,60 @@ from dbcat.catalog.models import (
     Job,
     JobExecution,
     JobExecutionStatus,
+    PiiTypes,
+    Task,
 )
-from dbcat.log_mixin import LogMixin
+
+logger = logging.getLogger("dbcat.Catalog")
 
 
-class Catalog(LogMixin):
-    def __init__(
-        self,
-        user: str,
-        password: str,
-        database: str,
-        host: str,
-        port: str = None,
-        **kwargs
-    ):
-        self.user: str = user
-        self.password: str = password
-        self.host: str = host
-        self.port: int = int(port) if port is not None else 5432
-        self.database: str = database
+class Catalog(ABC):
+    def __init__(self, **kwargs):
         self._engine: object = None
         self._scoped_session = None
         self._connection_args = kwargs
+        self._current_session: scoped_session = None
 
     @property
+    @abstractmethod
     def engine(self) -> object:
-        if self._engine is None:
-            self._engine = create_engine(
-                "postgresql://{user}:{password}@{host}:{port}/{database}".format(
-                    user=self.user,
-                    password=self.password,
-                    host=self.host,
-                    port=self.port,
-                    database=self.database,
-                ),
-                **self._connection_args
-            )
-        return self._engine
+        pass
 
-    @property
-    def scoped_session(self) -> scoped_session:
+    def get_scoped_session(self) -> scoped_session:
         if self._scoped_session is None:
-            Base.metadata.create_all(self.engine)
             self._scoped_session = scoped_session(sessionmaker(bind=self.engine))
 
         return self._scoped_session
 
-    def close(self):
-        if self._scoped_session is not None:
-            self._scoped_session.remove()
+    @property  # type: ignore
+    @contextmanager
+    def managed_session(self) -> scoped_session:
+        if self._current_session is not None:
+            yield self._current_session
+            return
 
+        try:
+            self._current_session = self.get_scoped_session()
+            logger.debug(
+                "Started new managed session: {}".format(self._current_session)
+            )
+            yield self._current_session
+        finally:
+            if self._current_session is not None:
+                logger.debug(
+                    "Removed managed session: {}".format(self._current_session)
+                )
+                self._current_session.remove()
+            self._current_session = None
+
+    def close(self):
         if self._engine is not None:
             self._engine.dispose()
 
     def _create(
         self, model, create_method="", create_method_kwargs=None, **kwargs
     ) -> Base:
-        session = self.scoped_session
+        session: scoped_session = self._current_session
 
         try:
             if create_method_kwargs is None:
@@ -85,8 +85,6 @@ class Catalog(LogMixin):
         except IntegrityError:
             session.rollback()
             return session.query(model).filter_by(**kwargs).one()
-        finally:
-            session.rollback()
 
     def add_source(self, name: str, source_type: str, **kwargs) -> CatSource:
         return self._create(CatSource, name=name, source_type=source_type, **kwargs)
@@ -145,16 +143,21 @@ class Catalog(LogMixin):
             create_method_kwargs={"context": context},
         )
 
+    def add_task(self, app_name: str, status: int, message: str) -> Task:
+        return self._create(
+            model=Task, app_name=app_name, status=status, message=message
+        )
+
     def get_source(self, source_name: str) -> CatSource:
         return (
-            self.scoped_session.query(CatSource)
+            self._current_session.query(CatSource)
             .filter(CatSource.name == source_name)
             .one()
         )
 
     def get_schema(self, source_name: str, schema_name: str) -> CatSchema:
         return (
-            self.scoped_session.query(CatSchema)
+            self._current_session.query(CatSchema)
             .join(CatSchema.source)
             .filter(CatSource.name == source_name)
             .filter(CatSchema.name == schema_name)
@@ -165,7 +168,7 @@ class Catalog(LogMixin):
         self, source_name: str, schema_name: str, table_name: str
     ) -> CatTable:
         return (
-            self.scoped_session.query(CatTable)
+            self._current_session.query(CatTable)
             .join(CatTable.schema)
             .join(CatSchema.source)
             .filter(CatSource.name == source_name)
@@ -175,10 +178,13 @@ class Catalog(LogMixin):
         )
 
     def get_columns_for_table(
-        self, table: CatTable, column_names: List[str] = None
+        self,
+        table: CatTable,
+        column_names: List[str] = None,
+        newer_than: TIMESTAMP = None,
     ) -> List[CatColumn]:
         stmt = (
-            self.scoped_session.query(CatColumn)
+            self._current_session.query(CatColumn)
             .join(CatColumn.table)
             .join(CatTable.schema)
             .join(CatSchema.source)
@@ -189,18 +195,20 @@ class Catalog(LogMixin):
 
         if column_names is not None:
             stmt = stmt.filter(CatColumn.name.in_(column_names))
+        if newer_than is not None:
+            stmt = stmt.filter(CatColumn.updated_at >= newer_than)
         stmt = stmt.order_by(CatColumn.sort_order)
         return stmt.all()
 
     def get_column(
-        self, database_name: str, schema_name: str, table_name: str, column_name: str
+        self, source_name: str, schema_name: str, table_name: str, column_name: str
     ) -> CatColumn:
         return (
-            self.scoped_session.query(CatColumn)
+            self._current_session.query(CatColumn)
             .join(CatColumn.table)
             .join(CatTable.schema)
             .join(CatSchema.source)
-            .filter(CatSource.name == database_name)
+            .filter(CatSource.name == source_name)
             .filter(CatSchema.name == schema_name)
             .filter(CatTable.name == table_name)
             .filter(CatColumn.name == column_name)
@@ -208,11 +216,11 @@ class Catalog(LogMixin):
         )
 
     def get_job(self, name: str) -> Job:
-        return self.scoped_session.query(Job).filter(Job.name == name).one()
+        return self._current_session.query(Job).filter(Job.name == name).one()
 
     def get_job_executions(self, job: Job) -> List[JobExecution]:
         return (
-            self.scoped_session.query(JobExecution)
+            self._current_session.query(JobExecution)
             .filter(JobExecution.job_id == job.id)
             .order_by(JobExecution.started_at.asc())
             .all()
@@ -220,31 +228,39 @@ class Catalog(LogMixin):
 
     def get_job_execution(self, job_execution_id: int) -> JobExecution:
         return (
-            self.scoped_session.query(JobExecution)
+            self._current_session.query(JobExecution)
             .filter(JobExecution.id == job_execution_id)
             .one()
         )
 
     def get_source_by_id(self, source_id: int) -> CatSource:
         return (
-            self.scoped_session.query(CatSource).filter(CatSource.id == source_id).one()
+            self._current_session.query(CatSource)
+            .filter(CatSource.id == source_id)
+            .one()
         )
 
     def get_schema_by_id(self, schema_id: int) -> CatSchema:
         return (
-            self.scoped_session.query(CatSchema).filter(CatSchema.id == schema_id).one()
+            self._current_session.query(CatSchema)
+            .filter(CatSchema.id == schema_id)
+            .one()
         )
 
     def get_table_by_id(self, table_id: int) -> CatTable:
-        return self.scoped_session.query(CatTable).filter(CatTable.id == table_id).one()
+        return (
+            self._current_session.query(CatTable).filter(CatTable.id == table_id).one()
+        )
 
     def get_column_by_id(self, column_id: int) -> CatColumn:
         return (
-            self.scoped_session.query(CatColumn).filter(CatColumn.id == column_id).one()
+            self._current_session.query(CatColumn)
+            .filter(CatColumn.id == column_id)
+            .one()
         )
 
     def get_job_by_id(self, job_id: int) -> Job:
-        return self.scoped_session.query(Job).filter(Job.id == job_id).one()
+        return self._current_session.query(Job).filter(Job.id == job_id).one()
 
     @staticmethod
     def _get_latest_job_executions(session: Session, job_ids: List[int]) -> Query:
@@ -258,21 +274,19 @@ class Catalog(LogMixin):
         query = (
             session.query(JobExecution.id)
             .filter(JobExecution.job_id.in_(job_ids))
-            .add_column(row_number_column)
+            .add_columns(row_number_column)
             .from_self()
             .filter(row_number_column == 1)
         )
-        query = query.from_self(JobExecution.id)
-        print(query)
-        return query
+        return query.from_self(JobExecution.id)
 
     def get_latest_job_executions(self, job_ids: List[int]) -> List[JobExecution]:
         return (
-            self.scoped_session.query(JobExecution)
+            self._current_session.query(JobExecution)
             .filter(
                 JobExecution.id.in_(
                     self._get_latest_job_executions(
-                        self.scoped_session(), job_ids
+                        self._current_session, job_ids
                     ).subquery()
                 )
             )
@@ -280,9 +294,9 @@ class Catalog(LogMixin):
         )
 
     def get_column_lineages(self, job_ids: List[int] = None) -> List[ColumnLineage]:
-        query = self.scoped_session.query(ColumnLineage)
+        query = self._current_session.query(ColumnLineage)
         if job_ids is not None and len(job_ids) > 0:
-            self.logger.debug(
+            logger.debug(
                 "Search for lineages from [{}]".format(
                     ",".join(str(v) for v in job_ids)
                 )
@@ -290,20 +304,40 @@ class Catalog(LogMixin):
             query = query.filter(
                 ColumnLineage.job_execution_id.in_(
                     self._get_latest_job_executions(
-                        self.scoped_session(), job_ids
+                        self.get_scoped_session(), job_ids
                     ).subquery()
                 )
             )
         else:
-            self.logger.debug("No job ids provided. Return all edges")
+            logger.debug("No job ids provided. Return all edges")
         return query.all()
 
     def get_sources(self) -> List[CatSource]:
-        return self.scoped_session.query(CatSource).all()
+        return self._current_session.query(CatSource).all()
+
+    def get_task_by_id(self, task_id: int) -> Task:
+        return self._current_session.query(Task).filter(Task.id == task_id).one()
+
+    def get_tasks_by_app_name(self, app_name: str) -> List[Task]:
+        return (
+            self._current_session.query(Task)
+            .filter(Task.app_name == app_name)
+            .order_by(Task.updated_at)
+            .all()
+        )
+
+    def get_latest_task(self, app_name: str) -> Task:
+        return (
+            self._current_session.query(Task)
+            .filter(Task.app_name == app_name)
+            .order_by(Task.updated_at.desc())
+            .limit(1)
+            .one_or_none()
+        )
 
     def search_sources(self, source_like: str) -> List[CatSource]:
         return (
-            self.scoped_session.query(CatSource)
+            self._current_session.query(CatSource)
             .filter(CatSource.name.like(source_like))
             .all()
         )
@@ -311,11 +345,11 @@ class Catalog(LogMixin):
     def search_schema(
         self, schema_like: str, source_like: Optional[str] = None
     ) -> List[CatSchema]:
-        stmt = self.scoped_session.query(CatSchema)
+        stmt = self._current_session.query(CatSchema)
         if source_like is not None:
             stmt = stmt.join(CatSchema.source).filter(CatSource.name.like(source_like))
         stmt = stmt.filter(CatSchema.name.like(schema_like))
-        self.logger.debug(str(stmt))
+        logger.debug(str(stmt))
         return stmt.all()
 
     def search_tables(
@@ -324,7 +358,7 @@ class Catalog(LogMixin):
         schema_like: Optional[str] = None,
         source_like: Optional[str] = None,
     ) -> List[CatTable]:
-        stmt = self.scoped_session.query(CatTable)
+        stmt = self._current_session.query(CatTable)
         if source_like is not None or schema_like is not None:
             stmt = stmt.join(CatTable.schema)
         if source_like is not None:
@@ -333,7 +367,7 @@ class Catalog(LogMixin):
             stmt = stmt.filter(CatSchema.name.like(schema_like))
 
         stmt = stmt.filter(CatTable.name.like(table_like))
-        self.logger.debug(str(stmt))
+        logger.debug(str(stmt))
         return stmt.all()
 
     def search_table(
@@ -359,7 +393,7 @@ class Catalog(LogMixin):
         schema_like: Optional[str] = None,
         source_like: Optional[str] = None,
     ) -> List[CatColumn]:
-        stmt = self.scoped_session.query(CatColumn)
+        stmt = self._current_session.query(CatColumn)
         if source_like is not None or schema_like is not None or table_like is not None:
             stmt = stmt.join(CatColumn.table)
         if source_like is not None or schema_like is not None:
@@ -372,7 +406,6 @@ class Catalog(LogMixin):
             stmt = stmt.filter(CatTable.name.like(table_like))
 
         stmt = stmt.filter(CatColumn.name.like(column_like))
-        self.logger.debug(str(stmt))
         return stmt.all()
 
     def update_source(
@@ -381,3 +414,55 @@ class Catalog(LogMixin):
         return self._create(
             DefaultSchema, source_id=source.id, schema_id=default_schema.id
         )
+
+    def set_column_pii_type(self, column: CatColumn, pii_type: PiiTypes):
+        column.pii_type = pii_type
+        self._current_session.commit()
+
+
+class PGCatalog(Catalog):
+    def __init__(
+        self,
+        user: str,
+        password: str,
+        database: str,
+        host: str,
+        port: str = None,
+        **kwargs
+    ):
+        super(PGCatalog, self).__init__(**kwargs)
+        self.user: str = user
+        self.password: str = password
+        self.host: str = host
+        self.port: int = int(port) if port is not None else 5432
+        self.database: str = database
+
+    @property
+    def engine(self) -> object:
+        if self._engine is None:
+            self._engine = create_engine(
+                "postgresql://{user}:{password}@{host}:{port}/{database}".format(
+                    user=self.user,
+                    password=self.password,
+                    host=self.host,
+                    port=self.port,
+                    database=self.database,
+                ),
+                **self._connection_args
+            )
+        return self._engine
+
+
+class SqliteCatalog(Catalog):
+    def __init__(self, path: str, **kwargs):
+        super(SqliteCatalog, self).__init__(**kwargs)
+        self.path: str = path
+
+    @property
+    def engine(self) -> object:
+        if self._engine is None:
+            logger.debug("SQLite path: {}".format(self.path))
+            self._engine = create_engine(
+                "sqlite:///{path}".format(path=self.path), **self._connection_args
+            )
+        return self._engine
