@@ -1,8 +1,10 @@
 import logging
+import re
 from contextlib import closing
-from typing import Any, Tuple, Type
+from typing import Any, Generator, List, Optional, Pattern, Tuple, Type
 
 from databuilder import Scoped
+from databuilder.extractor.athena_metadata_extractor import AthenaMetadataExtractor
 from databuilder.extractor.base_extractor import Extractor
 from databuilder.extractor.base_postgres_metadata_extractor import (
     BasePostgresMetadataExtractor,
@@ -21,10 +23,19 @@ from pyhocon import ConfigFactory, ConfigTree
 
 from dbcat.catalog.catalog import Catalog
 from dbcat.catalog.models import CatSource
+from dbcat.catalog.sqlite_extractor import SqliteMetadataExtractor
 
 
 class DbScanner:
-    def __init__(self, catalog: Catalog, source: CatSource):
+    def __init__(
+        self,
+        catalog: Catalog,
+        source: CatSource,
+        include_schema_regex_str: Optional[List[str]] = None,
+        exclude_schema_regex_str: Optional[List[str]] = None,
+        include_table_regex_str: Optional[List[str]] = None,
+        exclude_table_regex_str: Optional[List[str]] = None,
+    ):
         self._name = source.name
         self._extractor: Extractor
         self._conf: ConfigTree
@@ -41,14 +52,75 @@ class DbScanner:
         elif source.source_type == "redshift":
             self._extractor, self._conf = DbScanner._create_redshift_extractor(source)
         elif source.source_type == "snowflake":
-            self._extractor, self._conf = DbScanner._create_snowflake_extractors(source)
+            self._extractor, self._conf = DbScanner._create_snowflake_extractor(source)
+        elif source.source_type == "sqlite":
+            self._extractor, self._conf = DbScanner._create_sqlite_extractor(source)
+        elif source.source_type == "athena":
+            self._extractor, self._conf = DbScanner._create_athena_extractor(source)
         else:
             raise ValueError("{} is not supported".format(source.source_type))
 
+        self.include_schema_regex: Optional[List[Pattern]] = None
+        self.exclude_schema_regex: Optional[List[Pattern]] = None
+        self.include_table_regex: Optional[List[Pattern]] = None
+        self.exclude_table_regex: Optional[List[Pattern]] = None
+
+        if include_schema_regex_str is not None and len(include_schema_regex_str) > 0:
+            self.include_schema_regex = [
+                re.compile(exp, re.IGNORECASE) for exp in include_schema_regex_str
+            ]
+
+        if exclude_schema_regex_str is not None and len(exclude_schema_regex_str) > 0:
+            self.exclude_schema_regex = [
+                re.compile(exp, re.IGNORECASE) for exp in exclude_schema_regex_str
+            ]
+
+        if include_table_regex_str is not None and len(include_table_regex_str) > 0:
+            self.include_table_regex = [
+                re.compile(exp, re.IGNORECASE) for exp in include_table_regex_str
+            ]
+
+        if exclude_table_regex_str is not None and len(exclude_table_regex_str) > 0:
+            self.exclude_table_regex = [
+                re.compile(exp, re.IGNORECASE) for exp in exclude_table_regex_str
+            ]
+
     @property
     def name(self):
         return self._name
 
+    @staticmethod
+    def _test_regex(
+        name: str,
+        include_regex: Optional[List[Pattern]] = None,
+        exclude_regex: Optional[List[Pattern]] = None,
+    ) -> bool:
+
+        passed = False
+        if include_regex is not None and len(include_regex) > 0:
+            for regex in include_regex:
+                passed |= regex.search(name) is not None
+        else:
+            passed = True
+
+        if exclude_regex is not None and len(exclude_regex) > 0:
+            for regex in exclude_regex:
+                passed &= regex.search(name) is None
+
+        return passed
+
+    def _filter_rows(
+        self, extractor: Extractor
+    ) -> Generator[TableMetadata, None, None]:
+        while record := extractor.extract():
+            if DbScanner._test_regex(
+                record.schema, self.include_schema_regex, self.exclude_schema_regex
+            ) and DbScanner._test_regex(
+                record.name, self.include_table_regex, self.exclude_table_regex
+            ):
+                yield record
+        return None
+
     def scan(self):
         schema_count = 0
         table_count = 0
@@ -56,7 +128,7 @@ class DbScanner:
         with closing(self._extractor) as extractor:
             extractor.init(Scoped.get_scoped_conf(self._conf, extractor.get_scope()))
 
-            record: TableMetadata = extractor.extract()
+            record: TableMetadata = next(self._filter_rows(extractor))
             current_schema = self._catalog.add_schema(
                 schema_name=record.schema, source=self._source
             )
@@ -84,7 +156,10 @@ class DbScanner:
                     )
                     index += 1
                     column_count += 1
-                record = extractor.extract()
+                try:
+                    record = next(self._filter_rows(extractor))
+                except StopIteration:
+                    record = None
 
         logging.debug(
             "Scanned {} schemata, {} tables, {} columns".format(
@@ -106,7 +181,7 @@ class DbScanner:
             {
                 conn_string_key: source.conn_string,
                 f"{scope}.{extractorClass.CLUSTER_KEY}": source.cluster,
-                f"{scope}.{extractorClass.DATABASE_KEY}": source.name,
+                f"{scope}.{extractorClass.DATABASE_KEY}": source.database,
                 f"{scope}.{extractorClass.WHERE_CLAUSE_SUFFIX_KEY}": where_clause_suffix,
             }
         )
@@ -164,7 +239,7 @@ class DbScanner:
             {
                 conn_string_key: source.conn_string,
                 f"{scope}.{MysqlMetadataExtractor.CLUSTER_KEY}": source.cluster,
-                f"{scope}.{MysqlMetadataExtractor.DATABASE_KEY}": source.name,
+                f"{scope}.{MysqlMetadataExtractor.DATABASE_KEY}": source.database,
                 f"{scope}.{MysqlMetadataExtractor.WHERE_CLAUSE_SUFFIX_KEY}": where_clause_suffix,
             }
         )
@@ -174,7 +249,7 @@ class DbScanner:
     @staticmethod
     def _create_postgres_extractor(source: CatSource) -> Tuple[Extractor, Any]:
         where_clause_suffix = """
-            WHERE TABLE_SCHEMA NOT IN ('information_schema', 'pg_catalog')
+            st.schemaname NOT IN ('information_schema', 'pg_catalog', 'pg_toast')
         """
         return DbScanner._create_sqlalchemy_extractor(
             source, where_clause_suffix, PostgresMetadataExtractor
@@ -190,7 +265,7 @@ class DbScanner:
         )
 
     @staticmethod
-    def _create_snowflake_extractors(
+    def _create_snowflake_extractor(
         source: CatSource,
     ) -> Tuple[SnowflakeMetadataExtractor, Any]:
         extractor = SnowflakeMetadataExtractor()
@@ -201,9 +276,43 @@ class DbScanner:
             {
                 conn_string_key: source.conn_string,
                 f"{scope}.{SnowflakeMetadataExtractor.CLUSTER_KEY}": source.cluster,
-                f"{scope}.{SnowflakeMetadataExtractor.DATABASE_KEY}": source.name,
+                f"{scope}.{SnowflakeMetadataExtractor.DATABASE_KEY}": source.database,
+                f"{scope}.{SnowflakeMetadataExtractor.SNOWFLAKE_DATABASE_KEY}": source.database,
                 # f"{scope}.{SnowflakeMetadataExtractor.WHERE_CLAUSE_SUFFIX_KEY}": connection.where_clause_suffix,
             }
         )
 
         return extractor, conf
+
+    @staticmethod
+    def _create_athena_extractor(
+        source: CatSource,
+    ) -> Tuple[AthenaMetadataExtractor, Any]:
+        extractor = AthenaMetadataExtractor()
+        scope = extractor.get_scope()
+        conn_string_key = f"{scope}.{SQLAlchemyExtractor().get_scope()}.{SQLAlchemyExtractor.CONN_STRING}"
+
+        conf = ConfigFactory.from_dict(
+            {
+                conn_string_key: source.conn_string,
+                f"{scope}.{AthenaMetadataExtractor.CATALOG_KEY}": source.cluster,
+            }
+        )
+
+        return extractor, conf
+
+    @staticmethod
+    def _create_sqlite_extractor(
+        source: CatSource,
+    ) -> Tuple[SqliteMetadataExtractor, Any]:
+        extractor = SqliteMetadataExtractor()
+        scope = extractor.get_scope()
+        conn_string_key = f"{scope}.{SQLAlchemyExtractor().get_scope()}.{SQLAlchemyExtractor.CONN_STRING}"
+        conf = ConfigFactory.from_dict(
+            {
+                conn_string_key: source.conn_string,
+                f"{scope}.{SqliteMetadataExtractor.CLUSTER_KEY}": source.name,
+            }
+        )
+
+        return extractor, conf
