@@ -1,8 +1,11 @@
 import enum
 from typing import Optional
+from urllib.parse import quote_plus
 
+import sqlalchemy
 from snowflake.sqlalchemy import URL
 from sqlalchemy import (
+    JSON,
     TIMESTAMP,
     Column,
     Enum,
@@ -11,14 +14,21 @@ from sqlalchemy import (
     String,
     UniqueConstraint,
 )
-from sqlalchemy.dialects.postgresql import JSONB
 from sqlalchemy.ext.declarative import DeclarativeMeta, declarative_base
 from sqlalchemy.orm import relationship
+from sqlalchemy_mixins.repr import ReprMixin
+from sqlalchemy_mixins.serialize import SerializeMixin
+from sqlalchemy_mixins.timestamp import TimestampsMixin
 
 Base: DeclarativeMeta = declarative_base()
 
 
-class CatSource(Base):
+class BaseModel(Base, ReprMixin, SerializeMixin, TimestampsMixin):
+    __abstract__ = True
+    pass
+
+
+class CatSource(BaseModel):
     __tablename__ = "sources"
 
     id = Column(Integer, primary_key=True)
@@ -41,6 +51,10 @@ class CatSource(Base):
     account = Column(String)
     role = Column(String)
     warehouse = Column(String)
+    aws_access_key_id = Column(String)
+    aws_secret_access_key = Column(String)
+    region_name = Column(String)
+    s3_staging_dir = Column(String)
 
     schemata = relationship("CatSchema", back_populates="source")
     jobs = relationship("Job", back_populates="source")
@@ -72,6 +86,10 @@ class CatSource(Base):
         account: Optional[str] = None,
         role: Optional[str] = None,
         warehouse: Optional[str] = None,
+        aws_access_key_id: Optional[str] = None,
+        aws_secret_access_key: Optional[str] = None,
+        region_name: Optional[str] = None,
+        s3_staging_dir: Optional[str] = None,
         **kwargs,
     ):
         self.uri = uri
@@ -95,6 +113,10 @@ class CatSource(Base):
         self.role = role
         self.account = account
         self.warehouse = warehouse
+        self.aws_access_key_id = aws_access_key_id
+        self.aws_secret_access_key = aws_secret_access_key
+        self.region_name = region_name
+        self.s3_staging_dir = s3_staging_dir
 
     @property
     def conn_string(self):
@@ -110,6 +132,19 @@ class CatSource(Base):
                 warehouse=self.warehouse,
                 role=self.role,
             )
+        elif self.source_type == "sqlite":
+            conn_string = "sqlite:///{}".format(self.uri)
+        elif self.source_type == "athena":
+            conn_string = (
+                "awsathena+rest://{aws_access_key_id}:{aws_secret_access_key}@athena.{region_name}.amazonaws.com:443/"
+                "{schema_name}?s3_staging_dir={s3_staging_dir}".format(
+                    aws_access_key_id=quote_plus(self.aws_access_key_id),
+                    aws_secret_access_key=quote_plus(self.aws_secret_access_key),
+                    region_name=self.region,
+                    schema_name=self.database,
+                    s3_staging_dir=quote_plus(self.s3_staging_dir),
+                )
+            )
         else:
             username_password_placeholder = (
                 f"{self.username}:{self.password}" if self.password is not None else ""
@@ -145,7 +180,7 @@ class CatSource(Base):
         return hash(self.fqdn)
 
 
-class CatSchema(Base):
+class CatSchema(BaseModel):
     __tablename__ = "schemata"
 
     id = Column(Integer, primary_key=True)
@@ -170,7 +205,7 @@ class CatSchema(Base):
         return hash(self.fqdn)
 
 
-class DefaultSchema(Base):
+class DefaultSchema(BaseModel):
     __tablename__ = "default_schema"
 
     source_id = Column(Integer, ForeignKey("sources.id"), primary_key=True)
@@ -179,7 +214,7 @@ class DefaultSchema(Base):
     source = relationship("CatSource", back_populates="default_schema")
 
 
-class CatTable(Base):
+class CatTable(BaseModel):
     __tablename__ = "tables"
 
     id = Column(Integer, primary_key=True)
@@ -208,13 +243,35 @@ class CatTable(Base):
         return hash(self.fqdn)
 
 
-class CatColumn(Base):
+@enum.unique
+class PiiTypes(enum.Enum):
+    """PiiTypes enumerates the different types of PII data"""
+
+    NONE = enum.auto()
+    UNSUPPORTED = enum.auto()
+    PHONE = enum.auto()
+    EMAIL = enum.auto()
+    CREDIT_CARD = enum.auto()
+    ADDRESS = enum.auto()
+    PERSON = enum.auto()
+    LOCATION = enum.auto()
+    BIRTH_DATE = enum.auto()
+    GENDER = enum.auto()
+    NATIONALITY = enum.auto()
+    IP_ADDRESS = enum.auto()
+    SSN = enum.auto()
+    USER_NAME = enum.auto()
+    PASSWORD = enum.auto()
+
+
+class CatColumn(BaseModel):
     __tablename__ = "columns"
 
     id = Column(Integer, primary_key=True)
     name = Column(String)
     data_type = Column(String)
     sort_order = Column(Integer)
+    pii_type = Column(sqlalchemy.types.Enum(PiiTypes))
     table_id = Column(Integer, ForeignKey("tables.id"))
     table = relationship("CatTable", back_populates="columns", lazy="joined")
 
@@ -266,12 +323,12 @@ class CatColumn(Base):
         return hash(self.fqdn)
 
 
-class Job(Base):
+class Job(BaseModel):
     __tablename__ = "jobs"
 
     id = Column(Integer, primary_key=True)
     name = Column(String, unique=True)
-    context = Column(JSONB)
+    context = Column(JSON)
     source_id = Column(Integer, ForeignKey("sources.id"))
     source = relationship("CatSource", back_populates="jobs")
 
@@ -286,7 +343,7 @@ class JobExecutionStatus(enum.Enum):
     FAILURE = 2
 
 
-class JobExecution(Base):
+class JobExecution(BaseModel):
     __tablename__ = "job_executions"
 
     id = Column(Integer, primary_key=True)
@@ -304,11 +361,11 @@ class JobExecution(Base):
         )
 
 
-class ColumnLineage(Base):
+class ColumnLineage(BaseModel):
     __tablename__ = "column_lineage"
 
     id = Column(Integer, primary_key=True)
-    context = Column(JSONB)
+    context = Column(JSON)
 
     source_id = Column(Integer, ForeignKey("columns.id"))
     target_id = Column(Integer, ForeignKey("columns.id"))
@@ -330,3 +387,12 @@ class ColumnLineage(Base):
         return "<Edge: {} -> {} by {}. payload: {}>".format(
             self.source, self.target, self.job_execution, self.context
         )
+
+
+class Task(BaseModel):
+    __tablename__ = "tasks"
+
+    id = Column(Integer, primary_key=True)
+    app_name = Column(String)
+    status = Column(Integer)
+    message = Column(String)
